package JavaPonies;
/*
 * This is the form that holds the Pony animations and handles clicks and drags
 */

import javax.swing.*;
import java.awt.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

@SuppressWarnings("serial")
public class Pony_Form extends JDialog implements ActionListener, MouseMotionListener, MouseListener, KeyListener {
	// Pony image control
	public JLabel Pony_Image = new JLabel();
	
	// Pony name
	private String PonyName = "";
	
	// Control the pony with arrow keys? 
	public boolean ManualControl = false;
	
	// Arrow keys being pressed
	public boolean PonyUp = false;
	public boolean PonyDown = false;
	public boolean PonyLeft = false;
	public boolean PonyRight = false;
	
	// Shift key being pressed (walk faster!!)
	public boolean PonySpeed = false;
	
	// Direction
	public boolean PonyDirection = false;
	
	// Time that the Pony last spoke
	private Date LastSpoke = new Date();
	
	// List of lines to speak
	private List<String> Lines = new LinkedList<String>();
	
	// Tooltip window references
	private JWindow tooltip = new JWindow();
	private JLabel tooltiplabel = new JLabel();
	
	// Tooltip timer
	private Timer tooltiptimer;
	
	// Click menu
	private JPopupMenu popupmenu = new JPopupMenu();
	private JMenuItem manualcontrolmenu = null;
	
	// Constructor
	public Pony_Form() {
		// Initialize tooltip window
		initializeTooltip();
		
		// Initialize form
		initializeForm();
	}

	// Initialize the window controls, transparency and handlers
	private void initializeForm() {
		// Remove borders and window buttons
		this.setUndecorated(true);
		
		// Add the Pony image control to the form and make it take all the space
		FlowLayout newLayout = new FlowLayout();
		newLayout.setVgap(0);
		newLayout.setHgap(0);
		this.getContentPane().setLayout(newLayout);		
		this.getContentPane().add(Pony_Image, BorderLayout.CENTER);
		
		// Attempt to make the window fully transparent
		try {
		   Class<?> awtUtilitiesClass = Class.forName("com.sun.awt.AWTUtilities");
		   Method mSetWindowOpacity = awtUtilitiesClass.getMethod("setWindowOpaque", Window.class, boolean.class);
		   mSetWindowOpacity.invoke(null, this, Boolean.valueOf(false));
		} catch (NoSuchMethodException ex) {
		   ex.printStackTrace();
		} catch (SecurityException ex) {
		   ex.printStackTrace();
		} catch (ClassNotFoundException ex) {
		   ex.printStackTrace();
		} catch (IllegalAccessException ex) {
		   ex.printStackTrace();
		} catch (IllegalArgumentException ex) {
		   ex.printStackTrace();
		} catch (InvocationTargetException ex) {
		   ex.printStackTrace();
		}
		
		// Add various event handlers
		Pony_Image.setFocusable(true);
		Pony_Image.addKeyListener(this);
				
		Pony_Image.setDoubleBuffered(Options.EnableDoubleBuffering);
		Pony_Image.addMouseListener(this);
		Pony_Image.addMouseMotionListener(this);
		
		// Initialize the click menu
		JMenuItem singlequit = new JMenuItem("Quit");
		singlequit.setActionCommand("ClosePony");
		singlequit.addActionListener(this);
		JMenuItem allquit = new JMenuItem("Quit ALL");
		allquit.setActionCommand("CloseAll");
		allquit.addActionListener(this);
		manualcontrolmenu = new JMenuItem("Take Control");
		manualcontrolmenu.setActionCommand("ManualControl");
		manualcontrolmenu.addActionListener(this);
		JMenuItem showoptions = new JMenuItem("Show Options");
		showoptions.setActionCommand("ShowOptions");
		showoptions.addActionListener(this);
		
		popupmenu.add(singlequit);
		popupmenu.add(allquit);
		popupmenu.add(manualcontrolmenu);
		popupmenu.add(showoptions);
		
		// Display the window (which should be transparent) and make it stay in front of every other window
		this.setSize(100, 100);
		this.setAlwaysOnTop(true);
		this.setVisible(false);
	}

	// Initialize the tooltip window (for pony speech)
	private void initializeTooltip() {
		// Boring stuff, it's a label inside a borderless window that gets hidden by a timer, nuff said
		FlowLayout newLayout = new FlowLayout();
		newLayout.setVgap(0);
		newLayout.setHgap(10);
		tooltip.getContentPane().setLayout(newLayout);
		tooltip.getContentPane().add(tooltiplabel, BorderLayout.CENTER);
		tooltip.getContentPane().setBackground(Color.WHITE);
		tooltip.setSize(200, 25);
		tooltip.setAlwaysOnTop(true);
		tooltip.setVisible(false);
		tooltiptimer = new Timer(3000, this);
		tooltiptimer.stop();
		tooltiptimer.setActionCommand("HideTooltip");
	}
	
	// Set the Pony name
	public void SetPonyName(String name) {
		PonyName = name;
	}
	
	// Set the Pony speaking lines
	public void SetLines(List<String> speaking_lines) {
		Lines = speaking_lines;
	}
	
	// Get the Pony speaking lines
	public List<String> GetLines() {
		return Lines;
	}
	
	// Speak and say cute things!
	public void Pony_Speak(GraphicsDevice screen, String line) {
		if (screen == null) return;
		
		Rectangle screenbounds = screen.getDefaultConfiguration().getBounds();
		
		// Make sure we have lines to say and aren't being possessed
		if (!Lines.isEmpty() && !ManualControl) {
			// Verify if the user wants ponies to speak
			if (Options.Pony_Speak_Chance_Counter > 0) {
				// Only speak once every 3 seconds at most
				if ((new Date()).getTime() - LastSpoke.getTime() > 3000) {
					LastSpoke = new Date();
					
					// Select the line to speak
					int selection = WinMain.rand.nextInt(Lines.size());
					String line_to_speak = Lines.get(selection);
					if (line.length() > 0) line_to_speak = line;
					
					// Position and display the tooltip with the given line to speak
					tooltiplabel.setText(PonyName + ": \"" + line_to_speak + "\"");
					Point newLocation = new Point(this.getX() - 60, this.getY() - 10);
					newLocation.setLocation(Math.max(newLocation.x, screenbounds.getMinX()), Math.max(newLocation.y, screenbounds.getMinY()));
					tooltip.setLocation(newLocation);
					
					tooltip.setVisible(true);

					newLocation.setLocation(Math.min(newLocation.x, screenbounds.getMaxX() - tooltiplabel.getWidth() - 10), Math.min(newLocation.y, screenbounds.getMaxY() - tooltiplabel.getHeight()));
					tooltip.setLocation(newLocation);

					tooltip.setSize(tooltiplabel.getWidth() + 10, tooltiplabel.getHeight());
					tooltiptimer.start();
				}
			}
		}
	}

	// actionPerformed handler (timer and click menu)
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == "HideTooltip") { // Hide tooltip, timer says it's time to stop talking!
			tooltip.setVisible(false);
			tooltiptimer.stop();
		} else if (e.getActionCommand() == "ClosePony") { // Close this pony
			for (Pony pony : WinMain.currentInstance.Active_Ponies) {
				if (pony.Form == this) {
					WinMain.currentInstance.Active_Ponies.remove(pony);
					break;
				}
			}
			this.setVisible(false);
		} else if (e.getActionCommand() == "CloseAll") { // Close ALL ponies (application will quit automatically)
			while (!WinMain.currentInstance.Active_Ponies.isEmpty()) {
				WinMain.currentInstance.Active_Ponies.clear();
			}
		} else if (e.getActionCommand() == "ManualControl") { // Take or release control of this Pony
			ManualControl = !ManualControl;
			this.requestFocus();
			Pony_Image.requestFocus();
		} else if (e.getActionCommand() == "ShowOptions") { // Show the options form
			new Options(this);
		}
	}

	// Display click menu on click (duh)
	public void mouseClicked(MouseEvent e) {
		// Change the text of ManualControl depending on its current value
		if (ManualControl)
			manualcontrolmenu.setText("Release Control");
		else
			manualcontrolmenu.setText("Take Control");
		
		popupmenu.show(this, e.getX() - 20, e.getY() - 20);
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	// Drag the pony around like a big bully
	public void mouseDragged(MouseEvent e) {
		if (Options.Pony_Dragging_Enabled) {
			this.setLocation(new Point(e.getXOnScreen() - (this.getWidth() / 2), e.getYOnScreen() - (this.getHeight() / 2)));
		}
	}

	@Override
	public void mouseMoved(MouseEvent e) {		
	}

	// Make the Pony move against its will!
	public void keyPressed(KeyEvent e) {
		if (ManualControl) {
			switch (e.getKeyCode()) { // If an arrow key is pressed, set the flag to go in that direction!
				case KeyEvent.VK_DOWN:
					PonyDown = true;
					break;
				case KeyEvent.VK_UP:
					PonyUp = true;
					break;
				case KeyEvent.VK_RIGHT:
					PonyRight = true;
					break;
				case KeyEvent.VK_LEFT:
					PonyLeft = true;
					break;
			}
			
			// If the shift key is pressed, set the flag to make the Pony walk faster
			if (e.isShiftDown()) {
				PonySpeed = true;
			} else {
				PonySpeed = false;
			}
		}
	}

	// Same as above, but unset movement flags instead
	public void keyReleased(KeyEvent e) {
		if (ManualControl) {
			switch (e.getKeyCode()) { 
				case KeyEvent.VK_DOWN:
					PonyDown = false;
					break;
				case KeyEvent.VK_UP:
					PonyUp = false;
					break;
				case KeyEvent.VK_RIGHT:
					PonyRight = false;
					break;
				case KeyEvent.VK_LEFT:
					PonyLeft = false;
					break;
			}
			
			if (e.isShiftDown()) {
				PonySpeed = true;
			} else {
				PonySpeed = false;
			}
		}
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
	}
	
	// Redraw the whole window to blank
	public void paint(Graphics g) {
		if (Options.EnableFullRedraw) {
	        Graphics2D g2d = (Graphics2D) g;
	        int w = this.getWidth();
	        int h = this.getHeight();
	        g2d.setComposite(AlphaComposite.Clear);
	        g2d.fillRect(0, 0, w, h);
	        g2d.setComposite(AlphaComposite.Src);
	        g2d.setPaint(g2d.getBackground());
	        g2d.fillRect(0, 0, w, h);
		}
		
		super.paint(g);
	}
	
	public void update(Graphics g) {
		if (Options.EnableFullRedraw) {
			paint(g);
		} else {
			super.update(g);
		}
	}
}
