package JavaPonies;
/*
 * Desktop Ponies for Java
 * Written by Jirugameshi
 * 
 * Adapted from Desktop Ponies by Random Anonymous Pony (http://www.ponychan.net/chan/g/res/7033.html)
 * Ponies created by various authors
 * 
 * Inanimacy is Pinkie Pie!
 * Thanks to ryder356 for OSX testing
 */

/*
 * This is the main form that will be created on application start
 */

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.swing.*;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Scanner;


@SuppressWarnings("serial")
public class WinMain extends JFrame implements ActionListener {
	// First two numbers of version should match the Desktop Ponies version for which the features are the same
	// i.e. if we have all the features of Desktop Ponies 1.10, the version should be v1.10.xxxx
	// the minor version is just for small bug fixes specifically related to the Java port
	private static String version = "v1.13.0";

	// ge will be used to retrieve the list of monitors 
	public static GraphicsEnvironment ge; 
	// screens_to_use contains the GraphicsDevice objects representing the monitors to use
	public static GraphicsDevice[] screens_to_use;
	// MoveTimer performs the ticks for Pony movement and behavior selection
	public static Timer MoveTimer = null;
	// Static reference to the instance of WinMain
	public static WinMain currentInstance = null;
	// Random number generator
	public static Random rand = new Random();

	
	// Ponies selection panel
	public JPanel poniesPanel = new JPanel();
	
	// Start button reference
	private JButton btnStart = null; 
	
	// List of active Pony objects
	public List<Pony> Active_Ponies  = new LinkedList<Pony>();
	
	// List of active Effects
	public List<Effect_Form> Active_Effects = new LinkedList<Effect_Form>();
	
	// Start flag
	public static boolean Ponies_Have_Launched = false;
	
	// AllowedMoves enum
	public enum AllowedMoves {
		None,
		Horizontal_Only,
		Vertical_Only,
        Horizontal_Vertical,
        Diagonal_Only,
        Diagonal_Horizontal,
        Diagonal_Vertical,
        All,
        MouseOver
	}
	
	// Directions enum
	public enum Directions {
		top,
		bottom,
		left,
		right,
		bottom_right,
		bottom_left,
		top_right,
		top_left,
		center
	}
	
	// Behavior options
	public static final int BO_name = 1;
	public static final int BO_probability = 2;
	public static final int BO_max_duration = 3;
	public static final int BO_min_duration = 4;
	public static final int BO_speed = 5;
	public static final int BO_right_image_path = 6;
	public static final int BO_left_image_path = 7;
	public static final int BO_movement_type = 8;
	public static final int BO_linked_behavior = 9;
	public static final int BO_speaking_start = 10;
	public static final int BO_speaking_end = 11;
	public static final int BO_skip = 12;
	public static final int BO_xcoord = 13;
	public static final int BO_ycoord = 14;
	public static final int BO_object_to_follow = 15;
	
	// PROGRAM START POINT
	public static void main(String args[]) {
		// Create a new instance of WinMain() and set the static reference
		currentInstance = new WinMain();
		// Call load of options from Settings file
		Options.loadOptions();
	}
	
	// Constructor
	WinMain() {
		// Set title and version number
		super("Desktop Ponies for Java " + version);
		
		// Initialize controls and panels
		initializeWinMain();
		
		// Load ponies
		initializePonies();
	}

	// Load pony data from the directory and display the selection panels
	private void initializePonies() {
		// Start by getting a list of directories in running path
		File dir = new File(".");

		// Filter for only directories
		FileFilter fileFilter = new FileFilter() {
		    public boolean accept(File file) {
		        return file.isDirectory();
		    }
		};
		// Retrieve list of directories
		File[] files = dir.listFiles(fileFilter);

		// Loop through every directory
		for (File ponydir : files) {
			// Find the config file by filtering for Pony.ini
			FilenameFilter configfilter = new FilenameFilter() {
			    public boolean accept(File dir, String name) {
			        return name.equalsIgnoreCase("pony.ini");
			    }
			};
			// Retrieve the list of Pony.ini files (should be 0 or 1)
			File[] configfile = ponydir.listFiles(configfilter);

			// Only continue if the config file was found
			if (configfile.length > 0) {
				Scanner fi = null;
				
				// Attempt to open the Pony.ini file
				try {
					fi = new Scanner(new FileInputStream(configfile[0].getPath()));
				}
				catch (Exception e) {
					// For debug purposes
					System.out.println("Unable to open config file: " + configfile[0].getPath());
					System.out.println(e.toString());
				}
				
				// Initialize pony variables
				String PonyName = "";
				List<String> PonyLines = new LinkedList<String>();
				List<String> Behaviors = new LinkedList<String>();
				List<String> Effects = new LinkedList<String>();
				
				// Read pony.ini file
				while(fi.hasNextLine()) {
					// Retrieve line from file
					String line = fi.nextLine();
					
					// Ignore empty lines and lines that start with a single quote
					if (line.length() != 0 && !line.startsWith("'")) {
						// Split the values in the line (CSV)
						String[] columns = SplitWithQualifiers(line, ",", "\"");
						
						// Make sure we have at least one value to interpret
						if (columns.length > 0) {
							if (columns[0].equalsIgnoreCase("name")) { // Set pony name
								PonyName = columns[1].trim();
							} else if (columns[0].equalsIgnoreCase("behavior")) { // Add behavior line
								Behaviors.add(line);
							} else if (columns[0].equalsIgnoreCase("speak")) { // Add pony speak line
								PonyLines.add(columns[1].replace("\"", "")); // Get rid of any quotes in the speech
							} else if (columns[0].equalsIgnoreCase("effect")) {
								Effects.add(line);
							}
						}
					}
				}
				// Close the file
				fi.close();

				// Verify that the pony has a valid name
				if (PonyName.length() > 0) {
					// Create a new pony object
					Pony newPony = new Pony(PonyName, new Pony_Form(), PonyLines);
					
					// Loop through behaviors list
					for (String behavior : Behaviors) {
						// CSV split
						String[] columns = SplitWithQualifiers(behavior, ",", "\"");
						
						// Verify if we have an unexpected number of columns
						if (columns.length <= BO_movement_type || (columns.length > BO_linked_behavior && columns.length <= BO_object_to_follow)) {
							// Debug output
							System.out.println("Warning:  You are missing a required parameter for pony " + PonyName + " in behvaior:\n" + behavior);
						} else {
							// Initialize behavior variables
							AllowedMoves movement = AllowedMoves.None;
							String linked_behavior = "";
							String speak_start = "";
							String speak_end = "";
							int xcoord = 0;
							int ycoord = 0;
							String follow = "";
							
							boolean skip = false;
							
							// Set allowed movement
							if (columns.length > 0) {
								if (columns[BO_movement_type].trim().equalsIgnoreCase("none")) {
									movement = AllowedMoves.None;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("horizontal_only")) {
									movement = AllowedMoves.Horizontal_Only;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("vertical_only")) {
									movement = AllowedMoves.Vertical_Only;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("horizontal_vertical")) {
									movement = AllowedMoves.Horizontal_Vertical;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("diagonal_only")) {
									movement = AllowedMoves.Diagonal_Only;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("diagonal_horizontal")) {
									movement = AllowedMoves.Diagonal_Horizontal;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("diagonal_vertical")) {
									movement = AllowedMoves.Diagonal_Vertical;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("all")) {
									movement = AllowedMoves.All;
								} else if (columns[BO_movement_type].trim().equalsIgnoreCase("mouseover")) {
									movement = AllowedMoves.MouseOver;
								}
							}						
							
							// Verify if we have special behaviors
							if (columns.length > BO_linked_behavior) {
								linked_behavior = columns[BO_linked_behavior].trim();
								speak_start = columns[BO_speaking_start].trim();
								speak_end = columns[BO_speaking_end].trim();
								skip = Boolean.parseBoolean(columns[BO_skip].trim());
								xcoord = Integer.parseInt(columns[BO_xcoord].trim());
								ycoord = Integer.parseInt(columns[BO_ycoord].trim());
								follow = columns[BO_object_to_follow].trim();
							}
	
							// Add behavior values to the Pony object
	                        //                    name,     , Probabiliy, Max_Secs  , Min_Secs  , Speed     , image path, left image path, move_type, Linked behavior, speaking line_start, speaking line_end , skip normally unless processing links, x coord, ycoord, object to follow						
							newPony.Add_Behavior(columns[BO_name].trim(), Double.parseDouble(columns[BO_probability].trim()), Integer.parseInt(columns[BO_max_duration].trim()), Integer.parseInt(columns[BO_min_duration].trim()), Integer.parseInt(columns[BO_speed].trim()), ponydir.getPath() + ponydir.getPath().charAt(1) + columns[BO_right_image_path].trim(), ponydir.getPath() + ponydir.getPath().charAt(1) + columns[BO_left_image_path].trim(), movement, linked_behavior, speak_start, speak_end, skip, xcoord, ycoord, follow);
							
							// If the pony doesn't have a picture yet
							if (newPony.picture == null) {
								// Load a static image from this behavior
								BufferedImage image = null;
								try {
									image = ImageIO.read(new File(ponydir.getPath() + ponydir.getPath().charAt(1) + columns[BO_right_image_path].trim()));
								} catch(Exception e) {
									// Debug output
									System.out.println("Could not load image: " + ponydir.getPath() + ponydir.getPath().charAt(1) + columns[6].trim());
								}
								
								if (image != null) {
									// Set the pony's picture
									newPony.picture = new ImageIcon(image);
								}
							}
						}
					}
					
					// Loop through Effects list
					for (String effect : Effects) {
						// CSV split
						String[] columns = SplitWithQualifiers(effect, ",", "\"");
						
                        //1 = effect name
                        //2 = behvaior name
                        //3 = image
                        //4 = duration
                        //5 = delay before next
                        //6 = location relative to pony.
                        //7 = center of effect
                        //8 = effect follows pony
						
						boolean found_behavior = false;
						
						// Try to find the behavior to associate with
						for (Behavior behavior : newPony.Behaviors) {
							if (behavior.Name.equalsIgnoreCase(columns[2].replace('"', ' ').trim())) {
								Directions direction = Directions.center;
								Directions centering = Directions.center;
								
								try {
									direction = GetDirection(columns[6]);
									centering = GetDirection(columns[7]);
								} catch (Exception ex) {
									// Debug output
									System.out.println("Invalid placement direction or centering for effect " + columns[1] + " for pony " + PonyName + ":\n" + effect);
								}
								
						        // This is where we load the animation image
								Image image = null;
								try {
									image = Toolkit.getDefaultToolkit().createImage(ponydir.getPath() + ponydir.getPath().charAt(1) + columns[3].trim());
								} catch(Exception e) {
									// Debug output
									System.out.println("Could not load image: " + ponydir.getPath() + ponydir.getPath().charAt(1) + columns[3].trim());
									System.out.println(e.getMessage());
								}
								
								// Add the effect to the behavior if the image loaded correctly
								if (image != null) {
									behavior.AddEffect(columns[1].replace('"', ' ').trim(), new ImageIcon(image), Integer.parseInt(columns[4].trim()), Integer.parseInt(columns[5].trim()), direction, centering, Boolean.parseBoolean(columns[8].trim()));
									found_behavior = true;
								}
								break;
							}
						}
						
						if (!found_behavior) {
							// Debug output
							System.out.println("Could not find behavior for effect " + columns[1] + " for pony " + PonyName + ":\n" + effect);
						}
					}
					
					// Link special behaviors once done loading
					newPony.Link_Behaviors();
					
					// Make sure the pony has at least one correct behavior
					if (newPony.picture != null) {
						// Add the Pony object to the list of active ponies
						Active_Ponies.add(newPony);
						
						// Create a selection panel for this Pony
						Add_to_Menu(newPony);

						// Debug output
						System.out.println(PonyName + " successfully loaded.");
					} else {
						// Display an error message for this pony
						JOptionPane.showMessageDialog(this, "The pony " + PonyName + " could not be loaded. Make sure the Pony.ini file has all the required information.");
					}
				}
			}
		}
		
		// Verify if we have no ponies (the horror!!)
		if (Active_Ponies.isEmpty() == true) {
			JOptionPane.showMessageDialog(this, "Sorry, but you don't seem to have any ponies installed.  There should have at least been a 'Derpy' folder in the same spot as this program.");
			// Disable the start button (it would just immediately quit anyway)
			btnStart.setEnabled(false);
		}		
	}
	
	
	// Return the Directions value from the given string
	private Directions GetDirection(String setting) throws Exception {
		if (setting.trim().equalsIgnoreCase("top"))
			return Directions.top;
		if (setting.trim().equalsIgnoreCase("bottom"))
			return Directions.bottom;
		if (setting.trim().equalsIgnoreCase("left"))
			return Directions.left;
		if (setting.trim().equalsIgnoreCase("right"))
			return Directions.right;
		if (setting.trim().equalsIgnoreCase("bottom_right"))
			return Directions.bottom_right;
		if (setting.trim().equalsIgnoreCase("bottom_left"))
			return Directions.bottom_left;
		if (setting.trim().equalsIgnoreCase("top_right"))
			return Directions.top_right;
		if (setting.trim().equalsIgnoreCase("top_left"))
			return Directions.top_left;
		if (setting.trim().equalsIgnoreCase("center"))
			return Directions.center;
		
		// If not a valid direction, throw excepion
		throw new Exception("Invalid placement direction or centering for effect.");
	}

	// Initialize the window's components and member variables
	private void initializeWinMain() {
		// Retrieve the GraphicsEnvironment reference
		ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		// Initialize the list of monitors to use (all)
		screens_to_use = ge.getScreenDevices();
		
		// Initialize the main timer
		MoveTimer = new Timer(30, this);
		MoveTimer.stop();
		MoveTimer.setActionCommand("MoveTimer");
		
		// Close the application if X is pressed
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		// Panel for buttons
		JPanel buttonPanel = new JPanel();
		
		// Initialize the main menu buttons
		btnStart = new JButton("GIVE ME PONIES!");
		JButton btnZeroAll = new JButton("0 all ponies");
		JButton btnOptions = new JButton("OPTIONS");
		btnStart.setActionCommand("Start");
		btnStart.addActionListener(this);
		btnZeroAll.setActionCommand("ZeroAll");
		btnZeroAll.addActionListener(this);
		btnOptions.setActionCommand("Options");
		btnOptions.addActionListener(this);
		
		// Add the buttons to the button panel
		buttonPanel.add(btnZeroAll);
		buttonPanel.add(btnOptions);
		buttonPanel.add(btnStart);

		// Initialize the layout managers
		this.setLayout(new BorderLayout());
		poniesPanel.setLayout(new ModifiedFlowLayout());
		
		// Set the pony selection panel to be scrollable vertically
		JScrollPane js = new JScrollPane(poniesPanel);
		js.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		
		// Add the pony selection panel and buttons to the window
		this.add(js, BorderLayout.CENTER);
		this.add(buttonPanel, BorderLayout.SOUTH);
		
		// Display the window
		this.setSize(708, 537);
		this.setLocationRelativeTo(null);
		this.setVisible(true);
	}
	
	// Add a Pony selection panel
	private void Add_to_Menu(Pony pony) {
		// Initialize main and right panel 
		JPanel ponyselect = new JPanel();
		JPanel ponylabels = new JPanel();
		
		// Initialize the pony picture control
		JLabel ponypic = new JLabel();
		ponypic.setIcon(pony.picture);
		
		// Initialize the pony name and input controls
		JLabel ponyname = new JLabel(pony.Name);
		JLabel howmany = new JLabel("How many?");
		JNumField ponycount = new JNumField("1");
		
		// Add the label and input controls to the right panel
		ponylabels.setLayout(new GridLayout(4, 1));
		ponylabels.add(ponyname);
		ponylabels.add(new JLabel());
		ponylabels.add(howmany);
		ponylabels.add(ponycount);
		
		// Add the pony picture and right panel to the main panel
		ponyselect.setLayout(new GridLayout(1, 2));
		ponyselect.setBorder(BorderFactory.createLineBorder(Color.black, 1));
		ponyselect.add(ponypic);
		ponyselect.add(ponylabels);
		
		// Add the main panel to the pony selection panel
		poniesPanel.add(ponyselect);
		poniesPanel.getParent().validate(); //Redraw
	}

	// actionPerformed handler (buttons and main timer)
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("Start")) {
			go_Click();
		} else if (e.getActionCommand().equals("Options")) {
			options_Click();
		} else if (e.getActionCommand().equals("ZeroAll")) {
			zero_ponies_button_click();			
		} else if (e.getActionCommand().equals("MoveTimer")) {
			moveTimer_Tick();
		}
	}

	// Zero all of the pony selection values
	private void zero_ponies_button_click() {
		for (Component ponypanel : poniesPanel.getComponents()) {
			Component controls = ((JPanel)ponypanel).getComponent(1);
			Component ponycount = ((JPanel)controls).getComponent(3);
			JTextField textbox = (JTextField)ponycount;
			textbox.setText("0");
		}
	}

	// Display the options dialog
	private void options_Click() {
		new Options(this);
	}
	
	// Run the Desktop Ponies
	private void go_Click() {
		// Make sure we have some screens to use!
		if (screens_to_use.length == 0) {
			screens_to_use = ge.getScreenDevices();
		}
		
		// Retrieve the number of ponies to display from the pony selection panel
		List<Integer> number_of_ponies = new LinkedList<Integer>();
		int Total_Ponies = 0;
		
		// Loop through all of the pony selection panels
		for (Component ponypanel : poniesPanel.getComponents()) {
			Component controls = ((JPanel)ponypanel).getComponent(1); // Retrieve the right panel
			Component ponycount = ((JPanel)controls).getComponent(3); // Retrieve the input control
			JTextField textbox = (JTextField)ponycount; // Cast to JTextField
			
			// Get the pony count
			try {
				int numbertoadd = Integer.parseInt(textbox.getText());
				number_of_ponies.add(numbertoadd);
				Total_Ponies += numbertoadd;
			} catch(NumberFormatException ex) {
				number_of_ponies.add(0);
			}
		}
		
		// Verify if we have no ponies (omg!)
		if (Total_Ponies == 0) {
			JOptionPane.showMessageDialog(this, "The total is... no ponies...  That's TOO FEW PONY.");
			return;
		}

		// Make sure we don't have too many ponies
		if (Total_Ponies > Options.Max_Pony_Counter) {
			JOptionPane.showMessageDialog(this, "Sorry, I don't care who you are! " + Total_Ponies + " ponies is too much pony!  You'd only hurt yourself.\nNo, seriously your computer would crash.  Try less than 30 total.");
			return;
		}
		
		// Warn if we have a big crowd
		if (Total_Ponies > 20) {
			JOptionPane.showMessageDialog(this, Total_Ponies + " ponies!?!  I hope you can handle that much pony!  Here we go...");
		}
		
		// List of unselected ponies (the poor things)
		List<Pony> ponies_to_remove = new LinkedList<Pony>();
		
		// Loop through the selected ponies
		for (int i = 0; i < number_of_ponies.size(); i++) {
			if (number_of_ponies.get(i) == 0) {
				// Set unselected ponies to be removed
				ponies_to_remove.add(Active_Ponies.get(i));
			} else if (number_of_ponies.get(i) > 1) {
				// Duplicate any ponies with more than 1 selected
				for (int z = 1; z < number_of_ponies.get(i); z++) {
					Pony new_pony = Active_Ponies.get(i).Duplicate();
					Active_Ponies.add(new_pony);
				}
			}
		}
		
		// Remove the unselected ponies
		for (Pony pony : ponies_to_remove) {
			Active_Ponies.remove(pony);
		}
		
		// Start the simulation
		MoveTimer.start();
		
		// Hide this window
		this.setVisible(false);
		
		// HURRAY!
		Ponies_Have_Launched = true;
	}

	// Main timer tick
	private void moveTimer_Tick() {
		// List of ponies to remove in case of grave errors
		List<Pony> Ponies_to_Remove = new LinkedList<Pony>();
		
		// Loop through the active ponies
		for (Pony pony : Active_Ponies) {
			// Is the pony animation window invisible?
			if (pony.Form.isVisible() == false) {
				// Display the pony animation window
				try {
					pony.Form.setVisible(true);
					pony.Form.Pony_Image.setDoubleBuffered(Options.EnableDoubleBuffering);
					pony.Teleport();
				} catch (Exception ex) {
					Ponies_to_Remove.add(pony);
				}
			} else {
				// If already visible, perform normal pony things
				pony.Update();
			}
		}
		
		// Remove bugged ponies
		for (Pony pony : Ponies_to_Remove) {
			for (Effect_Form effect : pony.Active_Effects) {
				effect.setVisible(false);
			}
			Active_Ponies.remove(pony);
		}
		
		List<Effect_Form> effects_to_remove = new LinkedList<Effect_Form>();
		
		// Remove active effects that have run their course
		for (Effect_Form Effect : Active_Effects) {
			Effect.TimeLeft--;
			
			if (Effect.TimeLeft <= 0) {
				Effect.setVisible(false);
				effects_to_remove.add(Effect);
			}
		}
		
		// Also remove them from the list of active effects
		for (Effect_Form effect : effects_to_remove) {
			Active_Effects.remove(effect);
		}
		
		// If all ponies are gone, the program should quit
		if (Active_Ponies.isEmpty()) {
			System.exit(0);
		}
	}

	// SplitWithQualifiers function
	// Splits the string based on textdelimiter, but not if it is between textqualifiers
	public String[] SplitWithQualifiers(String SourceText, String TextDelimiter, String TextQualifier) {
		String[] strTemp;
		String[] strRes; int I; int J; String A; String B; boolean blnStart = false;
		B = "";
		
		if (TextDelimiter != " ") SourceText = SourceText.trim();
		strTemp = SourceText.split(TextDelimiter);
		for (I = 0; I < strTemp.length; I++) {
		    J = strTemp[I].indexOf(TextQualifier, 0);
		    if (J > -1) {
		        A = strTemp[I].replace(TextQualifier, "").trim();
		        if (strTemp[I].trim().equals(TextQualifier + A + TextQualifier)) {
		                B = B + A + " \n";
		                blnStart = false;
		        } else if (strTemp[I].trim().equals(TextQualifier + A)) {
		                B = B + A + TextDelimiter;
		                blnStart = true;
		        } else if (strTemp[I].trim().equals(A)) {
		                B = B + A + TextDelimiter;
		                blnStart = false;
		        } else if (strTemp[I].trim().equals(A + TextQualifier)) {
		                B = B + A + "\n";
		                blnStart = false;
		        }
		    } else {
		        if (blnStart)
		            B = B + strTemp[I] + TextDelimiter;
		        else
		            B = B + strTemp[I] + "\n";
		    }
		}
		if (B.length() > 0) {
		    B = B.substring(0, B.length());
		    strRes = B.split("\n");
		} else {
		    strRes = new String[1];
		    strRes[0] = SourceText;
		}
		return strRes;
	}
}